package algoritmy;

import entity.VektorAFitness;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;
import utility.DipUtils;
import utility.DipValidators;

/**
 * Trida realizujici Folding Counter (FC) algoritmus - vertikalni kompresi vektoru.
 * Poskytuje metody (vsechny dle definice FC) na vytvoreni dalsiho stavu od stavu
 * vychoziho, vytvoreni celkove FC sekvence delky=poctu bitu vektoru a pro slouceni
 * seedu.
 * 
 */

@SuppressWarnings("NEIMPLEMENTOVANO DO KONCE, POUZE DILCI FUNKCIONALITA")
public class AlgFoldingCounter {
       
    private String chybMaskaZal;      
    private Properties properties;
    private List<String> minMnozVektoru;

    public AlgFoldingCounter() {
        minMnozVektoru = new ArrayList<String>();
        try {
            properties = DipUtils.nactiProperties();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }


    /**
     * Metoda spoustici cely algoritmus.
     *     
     * @throws Exception
     */
    public void spustAlgFoldingCounter() throws Exception {
        init(properties);
        System.out.println("ODKOMENTUJ INIT!!!!");
        fcDlePatVektoru();
    }    

    /**
     * Metoda setridi a vrati sestupne serazene vektory. Vektory jsou serazene
     * dle poctu poruch, ktere kazdy vektor detekuje.
     *
     * @return setridene vektory
     * @throws Exception
     */
    public List<String> getSetrideneVektory() throws Exception {
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.mask.pat"));

        final String soubor = properties.getProperty("soubor.atalanta.pat");
        List<VektorAFitness> seznamVektoru = DipUtils.nactiDataZeSouboruJenVektory(soubor);
        List<VektorAFitness> razeneVektory = new ArrayList<VektorAFitness>();

        //od kazdeho vektoru chci jeho masku poruch, ty posleze setridim
        for (VektorAFitness vaf : seznamVektoru) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"),
                    vaf.getVektor(), false);
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("lzprMask.soubor.bat"));

            int detekuje = 0;
            detekuje = DipUtils.getPocetDetekPoruchZMasky
                    (DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")));            
            vaf = new VektorAFitness(vaf.getVektor(), detekuje);
            razeneVektory.add(vaf);
        }
        
        return nactiASeradVektory(razeneVektory);
    }

    /**
     * Metoda provadi algoritmus Folding counter. Jako seedy se pouzivaji setridene
     * vektory z pat souboru. Reseeding nastava po ukonceni kazde FC sekvence nad
     * danym vektorem.
     *
     * FC sekvence dle vektoru v pat souboru.
     *
     * @throws Exception
     */
    private void fcDlePatVektoru() throws Exception {
        DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
            properties.getProperty("generuj.testovaci.vektory.bat"));        

        int detekovatelnychPoruch = Integer.valueOf(properties.getProperty("detekovatelnych.poruch"));
        //int index = 0;

        List<String> setridenePat = getSetrideneVektory();
        List<String> minMnozina = new ArrayList<String>();
        String testVektorStart = setridenePat.remove(0);
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.pat"));

        minMnozina.add(testVektorStart);
        List<String> folCountSek = vytvorFoldingCounterSekv(testVektorStart);        
        String novySeed;        

        while (DipUtils.getPocetDetekPoruchZMasky(getChybMaskaZal()) != detekovatelnychPoruch) {
            //od daneho vektoru vytvori FC sekvenci a loguje po kazdem vektoru
            for (String testVektor : folCountSek) {                
                otestujVektor(testVektor);
                if (DipUtils.getPocetDetekPoruchZMasky(getChybMaskaZal()) == detekovatelnychPoruch) {
                    break;
                }
            }
            
            System.out.println("Size: " + setridenePat.size());
            folCountSek.clear();            
            logujProExcel(DipUtils.getPocetDetekPoruchZMasky(getChybMaskaZal()), 1, 999);
            novySeed = setridenePat.remove(0);
            minMnozina.add(novySeed);
            folCountSek = vytvorFoldingCounterSekv(novySeed);
            System.out.println("Reseeding!");
        }
        setMinMnozVektoru(minMnozina);
        DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), minMnozina, true);
    }


    /**
     * Metoda zjisti kolik poruch vektor detekuje, nastavi aktualni chybovou
     * masku, zaloguje prubeh a zapise vysledky do souboru.
     *
     * @param testVektor testovany vektor
     * @throws Exception
     */
    private void otestujVektor(String testVektor) throws Exception {
        String chybMaska = getChybMaskaZal();
        int aktDetPoruch = 0;

        DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"), testVektor, false);                
       
        DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
            properties.getProperty("lzprMask.soubor.bat"));

        chybMaska = DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska"));        
        
        chybMaskaZal = DipUtils.getVysledekOperaceOR(chybMaska, chybMaskaZal);
        setChybMaska(chybMaskaZal);
        aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
        DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);
        logujProExcel(aktDetPoruch, 0, 0);        
    }


    /**
     * Metoda nacte data (test. vektory) ze souboru a setridi je sestupne dle
     * poctu poruch, ktere je kazdy vektor schopen detekovat.
     *     
     * @param seznamVektoru
     * @return serazene vektory
     * @throws Exception
     */
    public List<String> nactiASeradVektory(List<VektorAFitness> seznamVektoru) throws Exception {
        List<String> serazeneVektory = new ArrayList<String>();        
        Collections.sort(seznamVektoru, new VektorAFitness());

        for (VektorAFitness vektorAFitness : seznamVektoru) {
            serazeneVektory.add(vektorAFitness.getVektor());
            System.out.println("Detekuje: " + vektorAFitness.getDetekPoruch());
        }
        return serazeneVektory;
    }

    /**
     * Metoda vybere a vrati jen nezbytne nutny pocet vektoru, ktere se stanou seedy,
     * takove, ktere svou sekvenci ve finale detekuji (pokryvaji) vsechny detekovatelne
     * poruchy obvodu.
     *
     * @param setrideneVektory vektory setridene sestupne dle poctu jednicek
     * @return seznam vektoru (seedu)
     */
    public List<String> getMinMnozinuSeedu(List<String> setrideneVektory) {
        int delkaPole = Integer.valueOf(DipUtils.getProperties().getProperty("stupen.polynomu"));
        int[] maskaPokryti = new int[delkaPole];
        int aktualnePokryto = 0;
        List<String> minMnozinaSeedu = new ArrayList<String>();

        for (String aktVektor : setrideneVektory) {
            aktualnePokryto = getPocetAktPokrytych(maskaPokryti);
            for (int i = 0; i < aktVektor.length(); i++) {
                if (aktVektor.charAt(i) == '1') {
                    maskaPokryti[i] = 1;
                }
            }
            if (getPocetAktPokrytych(maskaPokryti) - aktualnePokryto > 0) {
                minMnozinaSeedu.add(aktVektor);
                if (getPocetAktPokrytych(maskaPokryti) == delkaPole) {
                    return minMnozinaSeedu;
                }
            }
        }
        return minMnozinaSeedu;
    }

    /**
     * Metoda z masky pokryti zjisti a vrati pocet jiz detekovanych
     * (pokrytych) poruch.
     *
     * @param maskaPokryti
     * @return pocet aktualne pokrytych poruch
     */
    private int getPocetAktPokrytych(int[] maskaPokryti) {
        int pocetPokrytych = 0;
        for (int i = 0; i < maskaPokryti.length; i++) {
            if (maskaPokryti[i] == 1) {
                pocetPokrytych++;
            }
        }
        return pocetPokrytych;
    }   
    
    /**
     * Metoda vrati hodnotu 0, nebo 1 na adekvatni bitove pozici vektoru.
     *
     * @param bitPozice - i-ta pozice uvnitr vektoru
     * @param stavIndex - urcuje n-ty stav od stavu vychoziho
     * @return hodnota na bitove pozici "bitPozice", muze byt 0, nebo 1
     */
    private int getVysledekInverze(int bitPozice, int stavIndex) {
        if (bitPozice < stavIndex) {
            return bitPozice + 1;
        }
        return stavIndex;
    }

    /**
     * Metoda vezme aktualni stav a dle predpisu pro Folding Counter sekvenci vytvori
     * stav nasledny.
     *
     * @param aktStav
     * @param stavIndex
     * @return nasledny stav
     * @throws Exception
     */
    public String vytvorDalsiStav(String aktStav, int stavIndex) throws Exception {
        if (aktStav == null) {
            throw new Exception(DipUtils.getProperties().getProperty("nulovy.vychozi.stav.error"));
        }

        if (stavIndex < 0) {
            throw new Exception(DipUtils.getProperties().getProperty("index.error"));
        }

        aktStav = aktStav.toUpperCase();
        StringBuilder novyStav = new StringBuilder(aktStav);

        for (int i = 0; i < aktStav.length(); i++) {
            String aktZnak = aktStav.substring(i, i + 1);
            //pokud je lichy pocet negaci
            if (getVysledekInverze(i, stavIndex) % 2 != 0) {
                if (aktZnak.equals("1")) {
                    aktZnak = "0";
                } else if(aktZnak.equals("0")) {
                    aktZnak = "1";
                } else {
                    aktZnak = "X";
                }
            }
            novyStav.replace(i, i + 1, aktZnak);
        }
        return novyStav.toString();
    }

    /**
     * Metoda vrati list stavu (vektoru), ktere vznikly ze stavu (vektoru) vychoziho
     * generovanim Folding Counter sekvence.
     *
     * @param vychoziStav
     * @return folding counter sekvence stavu (vektoru)
     * @throws Exception
     */
    public List<String> vytvorFoldingCounterSekv(String vychoziStav) throws Exception {
        //System.out.println("\nVytvarim Folding counter sekvenci.");
        if (!DipValidators.jeParametrValidni(vychoziStav)) {
            throw new Exception(DipUtils.getProperties().getProperty("nulovy.vychozi.stav.error"));
        }

        vychoziStav = vychoziStav.toUpperCase();
        List<String> foldingCounterSek = new ArrayList<String>();
        foldingCounterSek.add(vychoziStav);

        for (int i = 1; i < vychoziStav.length() + 1; i++) {
            foldingCounterSek.add(vytvorDalsiStav(vychoziStav, i));
        }
        return foldingCounterSek;
    }    

    

    /**
     * Metoda vrati seed, ktery vznikl sloucenim dvou seedu. Jelikoz se tyto dva
     * lisi pouze v i-tem bitu, je prislusny i-ty bit nahrazen X (do not care symbolem).
     * Proto i v parametru metody uvazuji pouze jeden z dvojice seedu, nebot vim, jak
     * vypada ten druhy.
     *
     * @param seed
     * @param index
     * @return vysledek slouceni dvou seedu
     * @throws Exception
     */
    private String slucDvaSeedyFoldCount(String seed, int index) throws Exception {
        if (!DipValidators.jeParametrValidni(seed)) {
            throw new Exception(DipUtils.getProperties().getProperty("nulovy.vychozi.stav.error"));
        }

        seed = seed.toUpperCase();
        StringBuilder slouceny = new StringBuilder(seed);
        slouceny.replace(index, index+1, "X");
        return slouceny.toString();
    }

    /**
     * Metoda zredukuje sloucenim pocet seedu. Z kazde dvojice seedu vznikne jeden
     * vysledny a bit na adekvatni pozici je nahrazen X (do not care bit);
     *
     * @param puvodniSeedy
     * @return list sloucenych seedu
     * @throws Exception
     */
    public List<String> vratSlouceneSeedy(List<String> puvodniSeedy) throws Exception {
        System.out.println("\n\nSlucuji seedy.");
        List<String> slouceneSeedy = new ArrayList<String>();
        int index = puvodniSeedy.size()-1;

        if (puvodniSeedy == null || puvodniSeedy.size() == 0) {
            throw new Exception(DipUtils.getProperties().getProperty("nulovy.vychozi.stav.error"));
        }
         /* Mejme sest puvodnich seedu oznacenych 0...5. Slucovani by probihalo
         * 0-2, 4-(6); 1-3, 5-(7);
         * 4-5 (druha veta Teoremu)
         * Z dvojice menim jen prvni seed a jak je patrno, skacu o 4 indexy dale.
         */
        for (int j = 0; j < 2; j++) {
            for (int i = j; i < puvodniSeedy.size() - 2; i += 4) {
                slouceneSeedy.add(slucDvaSeedyFoldCount(puvodniSeedy.get(i), i));
            }
        }
        /* Je-li puvodni pocet seedu sudy, tak posledni dvojici sloucim tak, ze na
         * posledni bit dam X. Dle Teoremu Folding Counter.
         */
        if (puvodniSeedy.size() % 2 == 0) {
            slouceneSeedy.add(slucDvaSeedyFoldCount(puvodniSeedy.get(index), index-1));
        } else {
            /* Je-li puvodni pocet seedu lichy, tak posledni seed necham beze zmeny,
             * nebor ho nemam s cim slucovat.
             */
            slouceneSeedy.add(puvodniSeedy.get(index));
        }
        return slouceneSeedy;
    }
    
    //TODO dat do Utils!!!!
    /**
     *
     * @param initDetPoruch
     * @param typAkce 0-krok LZPR, 1-reseeding
     * @param hodnota
     * @throws Exception
     */
    private void logujProExcel(int initDetPoruch, int typAkce, int hodnota) throws Exception {
        if (typAkce == 0) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.poruchy.zaloha"),
                    String.valueOf(initDetPoruch) + ";" + "0", true);
        } else if (typAkce == 1) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.poruchy.zaloha"),
                    String.valueOf(initDetPoruch) + ";" + hodnota, true);
        }
    }

    
    /**
     * Metoda vyprazdni vsechny potrebne soubory.
     *
     * @param properties
     * @throws Exception
     */
    private void init(Properties properties) throws Exception {
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.rep"));
        //DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.pat"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.detekovane.poruchy"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.zaloha"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.prirustky"));
        setChybMaska(DipUtils.setDefaultChybMaska(Integer.valueOf(properties.getProperty("detekovatelnych.poruch"))));
    }
    
    //getters and setters
    public String getChybMaskaZal() {
        return chybMaskaZal;
    }

    public void setChybMaska(String chybMaskaZal) {
        this.chybMaskaZal = chybMaskaZal;
    }

    public List<String> getMinMnozVektoru() {
        return minMnozVektoru;
    }

    private void setMinMnozVektoru(List<String> minMnozVektoru) {
        this.minMnozVektoru = minMnozVektoru;
    }

}
