package algoritmy;

import entity.VektorAFitness;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import utility.DipUtils;

/**
 * Trida realizuje algoritmus folding counter.
 * Poskytuje dve hlavni metody - metodu pro nacteni a setrideni vektoru dle fitness
 * a metodu pro ziskani nejmensi sady seedu.
 */
@SuppressWarnings("NEIMPLEMENTOVANO DO KONCE, POUZE DILCI FUNKCIONALITA")
public class AlgFoldingCounterMinSeedu {

    private List<String> serazeneVektory = new ArrayList<String>();
    private List<String> minMnozinaSeedu = new ArrayList<String>();

    public AlgFoldingCounterMinSeedu() throws Exception {
        nactiASetridData(DipUtils.getProperties().getProperty("soubor.atalanta.pat"));
        getMinMnozinuSeedu();
    }


    /**
     * Metoda nacte data (test. vektory) ze souboru a setridi je sestupne dle
     * poctu poruch, ktere je kazdy vektor schopen detekovat.
     *
     * @param soubor
     * @throws Exception
     */
    public void nactiASetridData(String soubor) throws Exception {
        List<VektorAFitness> seznamVektoru = DipUtils.nactiDataZeSouboruJenVektory(soubor);
        Collections.sort(seznamVektoru, new VektorAFitness());

        for (VektorAFitness vektorAFitness : seznamVektoru) {
            serazeneVektory.add(vektorAFitness.getVektor());
        }        
    }

    /**
     * Metoda vybere a vrati jen nezbytne nutny pocet vektoru, ktere se stanou seedy,
     * takove, ktere svou sekvenci ve finale detekuji (pokryvaji) vsechny detekovatelne
     * poruchy obvodu.
     *
     * @return seznam vektoru (seedu)
     */
    public List<String> getMinMnozinuSeedu() {       
        int delkaPole = Integer.valueOf(DipUtils.getProperties().getProperty("stupen.polynomu"));
        int[] maskaPokryti = new int[delkaPole];      

        for (String aktVektor : serazeneVektory) {
            int aktualnePokryto = getPocetAktPokrytych(maskaPokryti);
            for (int i = 0; i < aktVektor.length(); i++) {
                if (aktVektor.charAt(i) == '1') {
                    maskaPokryti[i] = 1;
                }
            }
            if (getPocetAktPokrytych(maskaPokryti) - aktualnePokryto > 0) {
                minMnozinaSeedu.add(aktVektor);
                if (getPocetAktPokrytych(maskaPokryti) == delkaPole) {
                    return minMnozinaSeedu;
                }
            }
        }
        return minMnozinaSeedu;
    }

    /**
     * Metoda z masky pokryti zjisti a vrati pocet jiz detekovanych
     * (pokrytych) poruch.
     *
     * @param maskaPokryti
     * @return pocet aktualne pokrytych poruch
     */
    private int getPocetAktPokrytych(int[] maskaPokryti) {
        int pocetPokrytych = 0;
        for (int i = 0; i < maskaPokryti.length; i++) {
            if (maskaPokryti[i] == 1) {
                pocetPokrytych++;
            }
        }
        return pocetPokrytych;
    }

    /**
     * Metoda vrati minimalni mnozinu seedu.
     *
     * @return minimalni mnozina seedu
     */
    public List<String> getMinimalniMnozinuSeedu() {
        return minMnozinaSeedu;
    }
    
}