package algoritmy;

import java.util.Properties;
import utility.DipUtils;

/**
 * Trida zajistuje automatizovane provadeni algoritu Adaptivniho Jednopolynomialniho
 * Reseedingu (AJPOR).
 *
 */
public class AutomatizaceAJPOR {
    Properties properties = DipUtils.getProperties();

    public void spustAJPOR() throws Exception {
        final double konstanta = 0.1;
        final int pocetTestu = Integer.valueOf(properties.getProperty("pocet.vysledku"));

        for (int i = 1; i < pocetTestu; i++) {
            properties.setProperty("soubor.detekovane.poruchy",
                    properties.getProperty("soubor.detekovane.poruchy.zal"));
            properties.setProperty("soubor.detekovane.poruchy",
                    properties.getProperty("soubor.detekovane.poruchy.zal") + "_" + i);
        
        DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("generuj.testovaci.vektory.bat"));
                
        DipUtils.nactiVysledkyAtalanty(true);      
        init(properties);        
        
        int aktDetekPoruch = 0;
        int vychoziPocetDetPoruch = 0;
        int initPocetIteraci = Integer.valueOf(properties.getProperty("init.pocet.iteraci"));
        int ccaPocetIteraci = initPocetIteraci;
        int dilciPocetIter = 0;        

        try {
            int stupenPolynomu = Integer.valueOf(properties.getProperty("stupen.polynomu"));
            String testVektor = DipUtils.generujSeed(stupenPolynomu);
            String polynom = DipUtils.generujPolynom(stupenPolynomu);
            DipUtils.setPocetAktDetekovatPoruch("0");
            DipUtils.nastavPocDetekP = true;
            System.out.println("Seed: " + testVektor + " polynom: " + polynom);
            DipUtils.zapisSeedPolyDoSouboru("Seed: " + testVektor + " polynom: " + polynom);

            System.out.println(String.valueOf(aktDetekPoruch)+ " - " +(DipUtils.getMaxPocetDetekovatPoruch()));            

            while (!String.valueOf(aktDetekPoruch).equals(DipUtils.getMaxPocetDetekovatPoruch())) {
                //na zacatku kazdeho useku iteraci nastavim vychozi pocet detekovanych poruch
                if (vychoziPocetDetPoruch == 0) {
                    vychoziPocetDetPoruch = DipUtils.getAktPocetDetekPoruch();
                }                
                
                DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
                                
                DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                    properties.getProperty("lzpr.soubor.bat"));
                               
                DipUtils.nactiVysledkyAtalanty(false);
                
                DipUtils.zapisPocetDetekDoSouboru(DipUtils.getAktPocetDetekPoruch());
                testVektor = DipUtils.krokLZPR(polynom, testVektor);
                aktDetekPoruch = DipUtils.getAktPocetDetekPoruch();
                dilciPocetIter++;
                
                if (dilciPocetIter == Math.round(ccaPocetIteraci * konstanta)) {
                    //System.out.println("Akt. detek. poruch: " + aktDetekPoruch);

                    if ((aktDetekPoruch - vychoziPocetDetPoruch) < Math.round(aktDetekPoruch * konstanta)) {
                        //bude reseding
                        DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                            properties.getProperty("reseeding1.soubor.bat"));

                        String vektorReseedingu =
                                DipUtils.nactiRadekDat(properties.getProperty("soubor.nedetekovane.poruchy"));

                        while (vektorReseedingu == null) {
                            System.out.println("\n\n############NULOVY VEKTOR##############\n\n");
                            
                            vektorReseedingu =
                                DipUtils.nactiRadekDat(properties.getProperty("soubor.nedetekovane.poruchy"));
                        }                        
                        testVektor = vektorReseedingu;
                        //pokud se jedna o posledni vektor, delej jen kroky LZPR dokud nedetekujes zbyvajci poruchy
                        if (DipUtils.getPocetVektoruVSouboru(properties.getProperty("soubor.nedetekovane.poruchy")) == 1) {
                            System.out.println("###########\n\nJeden posledni vektor!\n\n###############");
                          
                            while (aktDetekPoruch < Integer.valueOf(properties.getProperty("detekovatelnych.poruch"))) {
                                DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
                                

                                DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                                    properties.getProperty("lzpr.soubor.bat"));
                                DipUtils.nactiVysledkyAtalanty(false);                               
                                DipUtils.zapisPocetDetekDoSouboru(DipUtils.getAktPocetDetekPoruch());                                

                                testVektor = DipUtils.krokLZPR(polynom, testVektor);
                                aktDetekPoruch = Integer.valueOf(DipUtils.getAktPocetDetekPoruch());
                            }
                            break;
                        }
                    } else {
                        //jinak dalsi LZPR krok.
                    }
                    ccaPocetIteraci += dilciPocetIter;
                    dilciPocetIter = 0;
                    vychoziPocetDetPoruch = 0;
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }       
       }
    }   

    private void init(Properties properties) throws Exception {       
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.rep"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.pat"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.detekovane.poruchy"));        
    }  
}
