package algoritmy;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import konsts.Konsts.Krok;
import utility.DipUtils;

/**
 * Trida poskytuje metody na srovnani vysledku dosazenych pri
 * ruznych algoritmech (a pristupech) na detekovani poruch.
 * Konkretne pro klasicke LZPR, CA (Celularni automat), 
 * deterministicky pristup a reseeding.
 * 
 */
public class Srovnani {
    private Properties properties;

    /**
     * Konstruktor tridy.
     * 
     * @throws Exception
     */
    public Srovnani() throws Exception {
        properties = DipUtils.nactiProperties();
        init(properties);
    }

    /**
     * Metoda simuluje krok LZPR, nebo krok s CA a pocet detekovanych poruch
     * zapisuje do souboru.
     *
     * @param varianta 
     * @param pocetKroku    
     * @throws Exception
     */
    public void simulujLZPRneboCA(String varianta, int pocetKroku) throws Exception {
        String polynom = properties.getProperty("polynom");
        String testVektor = properties.getProperty("seed");
        String chybMaskaZal = null;
        int aktDetPoruch = 0;                     

        for (int i = 0; i < pocetKroku; i++) {            
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"), testVektor, false);
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("lzprMask.soubor.bat"));

            chybMaskaZal = DipUtils.getVysledekOperaceOR(DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")),
                chybMaskaZal);
            aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
            DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);
            
            if(varianta.equalsIgnoreCase(Krok.LZPR.toString())) {
                testVektor = DipUtils.krokLZPR(polynom, testVektor);
                DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch, properties.getProperty("soubor.poruchy.lzpr"));
            } else if (varianta.equalsIgnoreCase(Krok.CA.toString())) {
                testVektor = DipUtils.getVektorPravidlo60(testVektor);
                DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch, properties.getProperty("soubor.poruchy.ca"));
            } else {
                throw new Exception(properties.getProperty("varianta.alg.error"));
            }

        }
    }

    /**
     * Metoda simuluje pouze deterministickou fazi, cili do testu
     * se uvazuji ouze vektory vygenerovane Atalantou.
     *
     * @throws Exception
     */
    public void simulujDeterFazi() throws Exception {
        String chybMaska = "";
        String chybMaskaZal = "";
        int aktDetPoruch = 0;
        List<Integer> detekPoruchy = new ArrayList<Integer>();

        DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("generuj.testovaci.vektory.bat"));

        DipUtils.nactiVysledkyAtalanty(true);
        FileReader reader = new FileReader(properties.getProperty("soubor.atalanta.pat"));
        BufferedReader bufferedReader = new BufferedReader(reader);
        String testVektor;

        while ((testVektor = bufferedReader.readLine()) != null) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"), testVektor, false);
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("lzprMask.soubor.bat"));
            chybMaska = DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska"));
            chybMaskaZal = DipUtils.getVysledekOperaceOR(chybMaska, chybMaskaZal);
            aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
            detekPoruchy.add(aktDetPoruch);            
        }

        reader.close();

        DipUtils.zapisPocetDetekDoSouboru(detekPoruchy, properties.getProperty("soubor.poruchy.determ"));
    }

        /**
     * Metoda vyprazdni vsechny potrebne soubory.
     *
     * @param properties
     * @throws Exception
     */
    private void init(Properties properties) throws Exception {
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.rep"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.pat"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.detekovane.poruchy"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.zaloha"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.prirustky"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.lzpr"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.ca"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.determ"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.reseed"));
    }
}
